{% extends "admin/change_form.html" %}
{% load i18n adminmedia admin_list sitetree %}

{% block after_related_objects %}
{% if change %}
	<style type="text/css">
		#result_list tbody tr:hover{
			background-color: #EDF3FE;
		}
		.align-center{
			text-align: center;
		}
	</style>
	
 <p>&nbsp;</p>
  <div class="module aligned">
  	<h2>{% trans "Site Tree Items" %} {{ original.alias }}</h2>
  </div>
	<p>&nbsp;</p>	
	{% if has_add_permission %}
	  <ul class="object-tools nofloat">
	    <li>
	      <a href="item_add/" class="addlink">
	        {% trans "Add Site Tree item" %}
	      </a>
	    </li>
	  </ul>
	{% endif %}
		
	<div class="module">
		<table cellspacing="0" id="result_list" width="100%">
			<thead>
				<tr>
					<th width="1%">{% trans "Hidden" %}</th>
					<th width="1%">{% trans "Menu" %}</th>
					<th width="1%">{% trans "Breadcrumbs" %}</th>
					<th width="1%">{% trans "Tree" %}</th>
					<th width="1%"><nobr>{% trans "Rights Restriction" %}</nobr></th>
					<th width="1%"><nobr>{% trans "For logged in" %}</nobr></th>
					<th width="25%">{% trans "Title" %}</th>
					<th width="20%">{% trans "URL" %}</th>
					<th>{% trans "Sort order" %}</th>
				</tr>
			</thead>
			<tbody>
				{% sitetree_tree from "original.alias" template "admin/sitetree/tree/tree.html" %}
			</tbody>
		</table>
	</div>
 <p>&nbsp;</p>
{% endif %}
{% endblock %}